@@ -5,7 +5,7 @@
 // Copyright (c) 2002 Rachel Hestilow
 //
 // This program is free software; you can redistribute it and/or
-// modify it under the terms of version 2 of the Lesser GNU General
+// modify it under the terms of version 2 of the Lesser GNU General 
 // Public License as published by the Free Software Foundation.
 //
 // This program is distributed in the hope that it will be useful,
@@ -24,58 +24,27 @@ namespace GLib {
 	using System.Collections;
 	using System.Runtime.InteropServices;
 	using GLib;
-
+	
 	internal class ManagedValue {
-
-		GCHandle gch;
-		object instance;
-		int ref_count = 1;
-
-		private ManagedValue (object instance)
-		{
-			this.instance = instance;
-			gch = GCHandle.Alloc (this);
-		}
-
-		IntPtr Handle {
-			get { return (IntPtr) gch; }
-		}
-
-		object Instance {
-			get { return instance; }
-		}
-
-		void Ref ()
-		{
-			ref_count++;
-		}
-
-		void Unref ()
-		{
-			if (--ref_count == 0) {
-				instance = null;
-				gch.Free ();
-			}
-		}
-
-		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
+		
+		[CDeclCallback]
 		delegate IntPtr CopyFunc (IntPtr gch);
-		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
+		[CDeclCallback]
 		delegate void FreeFunc (IntPtr gch);
-
+		
 		static CopyFunc copy;
 		static FreeFunc free;
 		static GType boxed_type = GType.Invalid;
 
-		[DllImport("libgobject-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport("libgobject-2.0-0.dll")]
 		static extern IntPtr g_boxed_type_register_static (IntPtr typename, CopyFunc copy_func, FreeFunc free_func);
-
+		
 		public static GType GType {
 			get {
 				if (boxed_type == GType.Invalid) {
 					copy = new CopyFunc (Copy);
 					free = new FreeFunc (Free);
-
+				
 					IntPtr name = Marshaller.StringToPtrGStrdup ("GtkSharpValue");
 					boxed_type = new GLib.GType (g_boxed_type_register_static (name, copy, free));
 					Marshaller.Free (name);
@@ -84,24 +53,14 @@ namespace GLib {
 				return boxed_type;
 			}
 		}
-
-		static ManagedValue FromHandle (IntPtr ptr)
-		{
-			GCHandle gch = (GCHandle) ptr;
-			ManagedValue val = gch.Target as ManagedValue;
-			if (val == null)
-				throw new Exception ("Unexpected GCHandle received.");
-			return val;
-		}
-
+		
 		static IntPtr Copy (IntPtr ptr)
 		{
 			try {
 				if (ptr == IntPtr.Zero)
 					return ptr;
-				ManagedValue val = FromHandle (ptr);
-				val.Ref ();
-				return ptr;
+				GCHandle gch = (GCHandle) ptr;
+				return (IntPtr) GCHandle.Alloc (gch.Target);
 			} catch (Exception e) {
 				ExceptionManager.RaiseUnhandledException (e, false);
 			}
@@ -114,8 +73,8 @@ namespace GLib {
 			try {
 				if (ptr == IntPtr.Zero)
 					return;
-				ManagedValue val = FromHandle (ptr);
-				val.Unref ();
+				GCHandle gch = (GCHandle) ptr;
+				gch.Free ();
 			} catch (Exception e) {
 				ExceptionManager.RaiseUnhandledException (e, false);
 			}
@@ -125,15 +84,14 @@ namespace GLib {
 		{
 			if (obj == null)
 				return IntPtr.Zero;
-			return new ManagedValue (obj).Handle;
+			return (IntPtr) GCHandle.Alloc (obj);
 		}
 
 		public static object ObjectForWrapper (IntPtr ptr)
 		{
 			if (ptr == IntPtr.Zero)
 				return null;
-			ManagedValue val = FromHandle (ptr);
-			return val == null ? null : val.Instance;
+			return ((GCHandle)ptr).Target;
 		}
 
 		public static void ReleaseWrapper (IntPtr ptr)
@@ -141,8 +99,8 @@ namespace GLib {
 			if (ptr == IntPtr.Zero)
 				return;
 
-			ManagedValue val = FromHandle (ptr);
-			val.Unref ();
+			((GCHandle)ptr).Free ();
 		}
 	}
 }
+
